<?php
/**
 * Geral\Repositories
 *
 * @category Php
 * @package  Geral\Repositories
 * @author   matthaeuscarvalho <matthaeus.carvalho@teknisa.com>
 * @license  www.teknisa.com Teknisa
 * @link     http://localhost:9090/tecfood
 */
namespace Geral\Repositories;

use Geral\Util\TratamentoDeMensagens;
use Setforms2\Shared\Registry;

/**
 * Class Produto
 *
 * @category Php
 * @package  Geral\Repositories
 * @author   matthaeuscarvalho <matthaeus.carvalho@teknisa.com>
 * @license  www.teknisa.com Teknisa
 * @link     http://localhost:9090/tecfood
 */
class Produto extends \Doctrine\ORM\EntityRepository
{
    /**
     * Executa Custo Produto
     *
     * @param String $filial    Filial
     * @param String $cdProduto Produto
     * @param String $tipoCusto Tipo de Custo
     * @param String $almox     Almoxerifado
     * @param String $local     Local
     * @param String $lote      Lote
     * @param String $subLote   SubLote
     * @param String $dtImpl    Data Impl
     * @param String $dtProc    Data Processamento
     *
     * @return bool|mixed
     */
    public function executaCustoProduto(
        $filial, $cdProduto, $tipoCusto, $almox, $local, $lote, $subLote, $dtImpl, $dtProc
    ) {
        $sql = "SELECT CUSTO_PRODUTO_LCEST(:P_CDFILIAL, :P_CDPRODUTO, :P_NRTIPOCUSTO, :P_DTREFERENCIA,
                                           :P_DTATUAL, :P_ALMOXARIFE, :P_LOCALESTOQ, :P_NRLOTEESTQ,
                                           :P_NRSUBLOTE) AS P_RESULT
                  FROM DUAL";

        $params = array(
            "P_CDFILIAL" => $filial,
            "P_CDPRODUTO" => $cdProduto,
            "P_NRTIPOCUSTO" => $tipoCusto,
            "P_DTREFERENCIA" => $dtImpl,
            "P_DTATUAL" => $dtProc,
            "P_ALMOXARIFE" => $almox,
            "P_LOCALESTOQ" => $local,
            "P_NRLOTEESTQ" => $lote,
            "P_NRSUBLOTE" => $subLote,
        );

        $entityManager = $this->getEntityManager();
        $result = $entityManager->getConnection()->fetchAll($sql, $params);
        return count($result) > 0 ? current($result) : false;
    }

    /**
     * Consulta Classificacao Serviço
     *
     * @param String $cdarvprod cdarvprod
     *
     * @return array
     * @throws \Exception
     */
    public function consultaClassificacaoServico($cdarvprod)
    {
        $sql = <<<SQL
        SELECT CDCLASSPROD
        FROM PRODUTO P,
            (SELECT MAX(CDARVPROD)  CDARVPROD
             FROM PRODUTO
             WHERE (INSTR(:CDARVPROD ,NVL(RTRIM(CDARVPROD), :CDARVPROD)) = 1)
               AND CDCLASSPROD IS NOT NULL  ) A
        WHERE
          A.CDARVPROD = P.CDARVPROD
SQL;
        $params = array(
            "CDARVPROD" => $cdarvprod
        );

        try{
            return \Setforms2\Shared\Connection::getInstance()->fetchAll($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }

    /**
     * Consulta Periodicidade do Produto
     *
     * @param String $cdfilsoli cdfilsoli
     * @param String $cdproduto Produto
     * @param String $nrorg     Nrorg
     *
     * @return null
     * @throws \Exception
     */
    public function consultaPeriodicidadeProduto($cdfilsoli, $cdproduto, $nrorg)
    {
        $sql = "
            SELECT MAX(PERFILPROD.CDPERIENTR) AS PERIODICIDADE
            FROM PRODUTO,
              PRODUTO PRODPERIENTR,
              PERFILPROD,
              PERIENTR
            WHERE PRODUTO.CDPRODESTO   = :P_PRODUTO
            AND PERFILPROD.CDPRODUTO   = PRODPERIENTR.CDPRODUTO
            AND PERFILPROD.CDFILIAL    = :P_CDFILSOLI
            AND PERFILPROD.CDPERIENTR  = PERIENTR.CDPERIENTR
            AND PERFILPROD.NRORG        = :P_NRORG
            AND PRODPERIENTR.CDARVPROD =
              (SELECT MAX(B.CDARVPROD)
              FROM PERFILPROD A,
                PRODUTO B
              WHERE A.CDPRODUTO                         = B.CDPRODUTO
              AND A.NRORG                               = :P_NRORG
              AND A.NRORG                               = B.NRORG
              AND A.CDFILIAL                            = :P_CDFILSOLI
              AND INSTR(PRODUTO.CDARVPROD, B.CDARVPROD) = 1
              )
        ";

        $paramsWhere = array(
            'P_PRODUTO'   => $cdproduto,
            'P_CDFILSOLI' => $cdfilsoli,
            'P_NRORG'     => $nrorg,
        );

        try {
            $this->_em->getConnection()->fetchAll($sql, $paramsWhere);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Consulta Produto Inativo
     *
     * @param String $nrorg     Nrorg
     * @param String $cdfilial  Filial
     * @param String $cdproduto Produto
     *
     * @return array
     * @throws \Exception
     */
    public function consultaProdutoInativo($nrorg, $cdfilial, $cdproduto)
    {
        $sql = "SELECT
                   PRODUTO.CDPRODUTO,
                   PRODFILI.IDPRODATIV
                FROM
                   PRODFILI,
                   PRODUTO
                WHERE
                   (PRODUTO.NRORG     =  :P_NRORG)           AND
                   (PRODUTO.CDPRODUTO =  :P_CDPRODUTO)         AND
                   (PRODFILI.CDFILIAL =  :P_CDFILIAL)        AND
                   (PRODUTO.CDPRODUTO = PRODFILI.CDPRODUTO)  AND
                   (PRODFILI.IDPRODATIV = 'I')
                   UNION

                SELECT
                   PRODUTO.CDPRODUTO,
                   PRODUTO.IDPRODATIVPR
                FROM
                   PRODUTO
                WHERE
                   (PRODUTO.NRORG     = :P_NRORG)            AND
                   (PRODUTO.CDPRODUTO = :P_CDPRODUTO)        AND
                   (PRODUTO.IDPRODATIVPR = 'N')";

        $paramsWhere = array(
            'P_NRORG'     => $nrorg,
            'P_CDFILIAL'  => $cdfilial,
            'P_CDPRODUTO'   => $cdproduto,
        );

        try {
            return $this->_em->getConnection()->fetchAll($sql, $paramsWhere);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Busca o último código Cdarvprod registrada
     *
     * @param String $nrorg          Nrorg
     * @param String $cdprodniveante Cdprodniveante
     *
     * @return array
     * @throws \Exception|Exception
     */
    public static function ultiArvProd($nrorg, $cdprodniveante)
    {
        $sql = "SELECT MAX(CDARVPROD) AS ULTCOD FROM PRODUTO WHERE CDPRODNIVANTE = '$cdprodniveante' AND NRORG = '$nrorg'";
        try {
            return \Setforms2\Runtime\Framework::getInstance()->fetchAll($sql);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Verifica Produto Inativo
     *
     * @param String $nrorg  Produto Inicial
     * @param String $prodini  Produto Inicial
     * @param String $prodfin  Produto Final
     * @param String $tproduto Todos Produtos
     *
     * @return array
     * @throws \Exception
     */
    public function verificaProdutoInativo($nrorg, $prodini, $prodfin, $tproduto)
    {
        $sql = "SELECT
                    PRODUTO.CDPRODUTO
                  FROM
                    PRODUTO
                 WHERE
                    PRODUTO.NRORG = :P_NRORG AND
                    (PRODUTO.IDPRODATIVPR = 'N' ) AND
                    ((PRODUTO.CDARVPROD BETWEEN :P_PRODINI AND :P_PRODFIN) OR (:P_TPRODUTO = 'T'))";

        $params = array(
            'P_PRODINI' => $prodini,
            'P_PRODFIN' => $prodfin,
            'P_TPRODUTO' => $tproduto,
            'P_NRORG' => $nrorg
        );

        try {
            return self::getEntityManager()->getConnection()->fetchArray($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function validaProdutoJaCadastrado($cdarvprod){
        $sql = "
            SELECT CDPRORIGREC
            FROM OPCPRODREC, PRODUTO
            WHERE
               CDPRODUTO = CDPRORIGREC AND
               :P_CDARVPROD LIKE CDARVPROD||'%'";
        $params = array(
            'P_CDARVPROD' => $cdarvprod
        );

        $entityManager = $this->getEntityManager();
        $result = $entityManager->getConnection()->fetchAll($sql, $params);
        return $result;


    }

    public function validaProdutoOpcionalJaCadastrado($cdarvprod){
        $sql = <<<SQL
            SELECT CDPRORIGREC, CDPROPCREC
            FROM OPCPRODREC, PRODUTO
            WHERE
               CDPRODUTO = CDPROPCREC AND
               :P_CDARVPROD LIKE CDARVPROD||'%'
SQL;
        $params = array(
            'P_CDARVPROD' => $cdarvprod
        );

        $entityManager = $this->getEntityManager();
        $result = $entityManager->getConnection()->fetchAll($sql, $params);
        return $result;
    }

    /**
     * Query que relaciona os produtos que serão enviados à Teknisa no processo de
     * Integração de Nutrientes.
     * As condições são que o produto possua IDTEMNUTRIENTE = S,
     * e que não existam nutrientes a ele relacionados, ou que o produto possua
     * IDALTERANUTR = S.
     *
     * @param String $nrorg Organização
     *
     * @return array.
     */
    public function sqlProdutosIntegracao($nrorg)
    {
        $sql = <<<SQL
           SELECT DISTINCT
               P.CDPRODUTO,
               P.CDARVPROD,
               P.NMPRODUTO,
               P.SGUNIDADE,
               (P.DSOBSNUTR) AS OBSERVACAO,
               P.VRPESOUNID,
               SYSDATE AS DATA
            FROM PRODUTO P
                 LEFT JOIN SIT_LOGPROD S ON (S.CDPRODUTO = P.CDPRODUTO) AND
                                            (S.DTRETORNO IS NULL) AND
                                            (S.NRORG = P.NRORG)
                 LEFT JOIN (SELECT CDPRODUTO, MAX(1) AS EXIST_PROD
                              FROM NUTRPROD
                              WHERE NUTRPROD.NRORG = '$nrorg'
                             GROUP BY CDPRODUTO) N ON N.CDPRODUTO = P.CDPRODUTO
            WHERE
               (P.IDTEMNUTRIENTE = 'S') AND
               (S.CDPRODUTO IS NULL) AND
               ((N.CDPRODUTO IS NULL) OR (IDALTERANUTR = 'S')) AND
               (P.IDPRODATIVPR = 'S')
               AND P.CDPRODUTO = P.CDPRODESTO
               AND P.NRORG = '$nrorg'
            ORDER BY
               P.NMPRODUTO
SQL;
        //
        $entityManager = Registry::getEntityManager();
        $result = $entityManager->getConnection()->fetchAll($sql);
        return $result;
    }

    /**
     * Query que relaciona os produtos que serão reenviados à Teknisa no processo de
     * Integração de Nutrientes.
     * Os produtos buscados são aqueles das remessas pendentes.
     * As condições são que o produto possua IDTEMNUTRIENTE = S,
     * e que não existam nutrientes a ele relacionados, ou que o produto possua
     * IDALTERANUTR = S.
     *
     * @param String $nrorg     Organização
     * @param String $nrremessa Remessa Pendente
     *
     * @return array.
     */
    public function sqlProdutosRemessaPendente ($nrorg, $nrremessa)
    {
        $sql = <<<SQL
           SELECT *
            FROM PRODUTO
            JOIN SIT_LOGPROD
            ON SIT_LOGPROD.NRORG = PRODUTO.NRORG
            AND SIT_LOGPROD.CDPRODUTO = PRODUTO.CDPRODUTO
            WHERE PRODUTO.NRORG = '$nrorg'
            AND SIT_LOGPROD.NRREMESSA = '$nrremessa'
            ORDER BY PRODUTO.NMPRODUTO
SQL;
        //
        $entityManager = Registry::getEntityManager();
        $result = $entityManager->getConnection()->fetchAll($sql);
        return $result;
    }

    public function validaCdarvprod($nrorg, $cdarvprod){
        $sql = <<<SQL
          SELECT CDARVPROD
            FROM PRODUTO
           WHERE NRORG = :P_NRORG
             AND CDARVPROD = :P_CDARVPROD
SQL;
        $params = array(
            'P_NRORG' => $nrorg,
            'P_CDARVPROD' => $cdarvprod
        );

        return $this->getEntityManager()->getConnection()->fetchAll($sql, $params);

    }
}