<?php

/* RegistroGRRF
 * @package    Logic
 * @subpackage Registro
 * @author     francismararaujo - 22/02/2013
 */
class RegistroGRRF extends FPARegistro{

    const SIM = 'S';
    const NAO = 'N';

    //usado no logic GRRF.php
    const CNPJ = '1';
    const CEI = '2';
    const CPF = '3';

    //registros contendo informacoes sobre empresa e tomador de servico.
    const LISTA_TIPO_REG_EMP = '10,12,13,14,20,21,50,51';

    //registros contendo informacoes sobre funcionarios.
    const LISTA_TIPO_REG_TRAB = '30, 32';

    const GRRF_MES_ANTERIOR_RESCISAO = 1;
    const GRRF_MES_RESCISAO = 2;
    const GRRF_AVISO_PREVIO_INDENIZADO = 3;
    const GRRF_PENSAO_JURIDICA = 4;
    const GRRF_SALDO_FINS_RESCISORIOS = 5;

    //const GRRF_SALARIO_FAMILIA = 1;
    //const GRRF_SALARIO_MATERNIDADE = 2;
    //const GRRF_REM_SEM_13SAL = 3;
    //const GRRF_13SAL_PAGO_MES = 4;
    //const GRRF_REM_BASE_GPS = 5;
    //const GRRF_REM_13SAL_MOV = 6;
    //const GRRF_REM_13SAL_GPS = 7;
    //const GRRF_INSS_MULT_VINCULOS = 8;

    private $_modalidade;
    private $_codigoRecolhimento;
    private $_competencia;
    private $_competencia13;

    /* Gera uma chave - Parte comum aos diversos registros.
     * @param string $tpRegistro  Numero do Tipo de Registro.
     * @param string $tpInscricao Numero do Tipo de Inscricao.
     * @param string $nrInscricao Numero da Inscricao.
     * @return void
     * @see FPARegistro::geraRegistro()
     * @author francismararaujo - 22/02/2013
     */

    protected function geraRegistro($tpRegistro, $tpInscricao, $nrInscricao){

        //bloco 1 | 001-002 | Tipo do Registro.
        $this->atribuiDado($tpRegistro, 2);

        //bloco 2 | 003 | Tipo de Inscricao.
        //@todo para empregador so pode aceitar 2.
        $this->atribuiDado($tpInscricao, 1);

        //bloco 3 | 004-017 | Inscricao da Empresa.
        $this->atribuiDado($nrInscricao, 14, parent::CARACTER_ZERO, self::PREENCHE_ESQUERDA);

    }

    /* Atribui valor a modalidade GFIP.
     * @param string $modalidade Identificador de modalidade de geracao do GFIP.
     * @return void
     * @author alessandrobartels
     */

    public function setModalidade($modalidade){
        $this->_modalidade = $modalidade;
    }

    /* Pega valor da modalidade GFIP.
     * @return string
     * @author alessandrobartels
     */

    public function getModalidade(){
        return $this->_modalidade;
    }

    /* Atribui valor a codigo de recolhimento GFIP.
     * @param string $codigo Codigo de recolhimento GFIP.
     * @return void
     * @author alessandrobartels
     */

    public function setCodigoRecolhimento($codigo){
        $this->_codigoRecolhimento = $codigo;
    }

    /* Pega codigo GFIP.
     * @return string
     * @author alessandrobartels
     */

    public function getCodigoRecolhimento(){
        return $this->_codigoRecolhimento;
    }

    /* Atribui competencia do GFIP.
     * @param TkDateTime $competencia Competencia (data) do GFIP.
     * @return void
     * @author alessandrobartels
     */

    public function setCompetencia(TkDateTime $competencia){
        $this->_competencia = $competencia->toString(TkDateTime::YEAR.TkDateTime::MONTH);
    }

    /* Pega a competencia do GFIP.
     * @return string
     * @author alessandrobartels
     */

    public function getCompetencia(){
        return $this->_competencia;
    }

    /* Atribui identificador de geracao da competencia 13.
     * @param boolean $competencia13 Identificador de geracao da competencia 13.
     * @return void
     * @author alessandrobartels
     */

    public function setCompetencia13($competencia13){
        $this->_competencia13 = $competencia13;
    }

    /* Pega identificador de competencia 13.
     * @return boolean
     * @author alessandrobartels
     */

    public function getCompetencia13(){
        return $this->_competencia13;
    }

}